/*The example code involves the following conditions.
	CCLK = 400 MHz, DCLK = 200 MHz, SYSCLK = 200MHz, SCLK0 = SCLK1 = 100 MHz
	SCB 3:
		- EPPI0: 24-bit transmit, internal clock and frame sync mode at EPPICLK = 50 MHz. Required throughput = 50 MHz * 3 = 150 MBytes/sec.
	SCB 4 and 5:
		- MDMA1 channels (18-19) and MDMA2 channels (20-21)
		- Throughout requirements depend upon the corresponding DMAx_BWLCNT register values. If not programmed, the MDMA channels request for the bandwidth with full throttle (every SCLK1 cycle). This means that both MDMA channels request from the SCB4 and SCB5 at individual throughput of 200 MBytes/sec * 4 = 800 MBytes/sec.

The code can be run under different conditions by commenting/uncommenting the corresponding conditions/optimization techniques in the "main.h" file.
*/
#include "main.h"

volatile clock_t clock_start_mdma1=0;
volatile clock_t clock_stop_mdma1=0;
volatile clock_t cycles_mdma1=0;
volatile clock_t cycles_mdma1_pre;
float throughput_mdma1=0;

volatile clock_t clock_start_mdma2;
volatile clock_t clock_stop_mdma2=0;
volatile clock_t cycles_mdma2=0;
volatile clock_t cycles_mdma2_pre;
float throughput_mdma2=0;

volatile clock_t clock_start_ppi0;
volatile clock_t clock_stop_ppi0;
volatile clock_t cycles_ppi0;
volatile clock_t cycles_ppi0_pre;
float throughput_ppi0=0;

float throughput_total=0;

#pragma align 32
section("L1_data_a") unsigned char ucIntSrcBuff[MAX_SIZE];

#pragma align 32
section("L1_data_a") unsigned char ucIntDest1Buff[MAX_SIZE];

#pragma align 32
section("L1_data_b") unsigned char ucIntDestBuff[MAX_SIZE];

#pragma align 32
section("L1_data_b") unsigned char ucIntSrc1Buff[MAX_SIZE];

void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg);
void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg);

int DMA_Count_PPI0=0;
int DMA_Count_MDMA1=0;
int DMA_Count_MDMA2=0;

int Error_Count_PPI0=0;


int main(void)
{
	unsigned int uiBytes=0; int i;

	//Enable the interrupts globally
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;

	//Enable MDMA1/2 streams as secure master
	*pREG_SPU0_SECUREP53=0x3;
	*pREG_SPU0_SECUREP54=0x3;
	*pREG_SPU0_SECUREP55=0x3;
	*pREG_SPU0_SECUREP56=0x3;

	//Work unit size = 4 pages = 8K words
	uiBytes=2048*4;

	//Program clock domain crossing registers in SYNC mode
	*pREG_SCB0_MST30_SYNC=0;
	*pREG_SCB0_MST31_SYNC=0;
	*pREG_SCB0_IB7_SYNC=0;
/*
	*pREG_SCB0_MST23_RQOS=15;
	*pREG_SCB0_MST23_WQOS=15;
	*pREG_SCB0_MST24_RQOS=15;
	*pREG_SCB0_MST24_WQOS=15;

	*pREG_SCB0_MST25_RQOS=10;
	*pREG_SCB0_MST25_WQOS=10;
	*pREG_SCB0_MST26_RQOS=10;
	*pREG_SCB0_MST26_WQOS=10;

*/
	//Enables PPI0 DMA
		#ifdef ENABLE_PPI0_DMA_187_5MBPS
			adi_int_InstallHandler(INTR_EPPI0_CH0_DMA, EPPI0_Interrupt_Handler, NULL, true);
			PDMA_CONFIG(29,PPI0_DMA_CFG, PPI0_DMA_DMC_START_ADDRESS, PPI0_MSIZE, uiBytes/PPI0_MSIZE);
			for(i=0;i<1000;i++);
			EPPI_init(0,uiBytes,EPPI0_CTL_CFG, EPPI0_CTL2_CFG );
		#endif



	//Enables PPI0 DMA
	#ifdef ENABLE_PPI0_DMA_150_MBPS
		adi_int_InstallHandler(INTR_EPPI0_CH0_DMA, EPPI0_Interrupt_Handler, NULL, true);
		PDMA_CONFIG(14,PPI0_DMA_CFG, PPI0_DMA_DMC_START_ADDRESS, PPI0_MSIZE, uiBytes/PPI0_MSIZE);
		for(i=0;i<1000;i++);
		EPPI_init(0,uiBytes,EPPI0_CTL_CFG, EPPI0_CTL2_CFG );
	#endif

	//Enables MDMA1
	#ifdef ENABLE_MDMA1_DMA_800MBPS
		adi_int_InstallHandler(INTR_MDMA1_DST, MDMA1_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(18, SOURCE_DMA_CFG_MDMA1 , MDMA1_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA1, uiBytes/SOURCE_MSIZE_MDMA1,19,DEST_DMA_CFG_MDMA1, ucIntDestBuff, DEST_MSIZE_MDMA1, uiBytes/DEST_MSIZE_MDMA1);

		#ifdef LIMIT_MDMA1_BW
			*pREG_DMA19_BWLCNT=SOURCE_MSIZE_MDMA0*SCLK1_SPEED/LIMIT_MDMA1_BW;//2048;
		#endif

	#endif

	//Enables MDMA2
	#ifdef ENABLE_MDMA2_DMA_800MBPS
		adi_int_InstallHandler(INTR_MDMA2_DST, MDMA2_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(20, SOURCE_DMA_CFG_MDMA2 , MDMA2_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA2, uiBytes/SOURCE_MSIZE_MDMA2,21,DEST_DMA_CFG_MDMA2, ucIntDest1Buff, DEST_MSIZE_MDMA2, uiBytes/DEST_MSIZE_MDMA2);

		#ifdef LIMIT_MDMA2_BW
			*pREG_DMA20_BWLCNT=SOURCE_MSIZE_MDMA2*SCLK1_SPEED/LIMIT_MDMA2_BW;//2048;
		#endif

	#endif

	//Wait for all the DMAs to complete at least 500 times
	#ifdef ENABLE_PPI0_DMA_187_5MBPS
		while(DMA_Count_PPI0<500);
	#endif

	#ifdef ENABLE_MDMA1_DMA_800MBPS
		while(DMA_Count_MDMA1<500);
	#endif

	#ifdef ENABLE_MDMA2_DMA_800MBPS
		while(DMA_Count_MDMA2<500);
	#endif

	//Check for any PPI underflow errors
	if(*pREG_EPPI0_STAT&ENUM_EPPI_STAT_YFIFOERR)
			Error_Count_PPI0=1;

	//Disable the peripherals and MDMAs
	*pREG_EPPI0_CTL =0;
	DISABLE_MDMAs(18,19);
	DISABLE_MDMAs(20,21);

	//Print the throughput numbers
	#ifdef ENABLE_PPI0_DMA_150_MBPS
		if(Error_Count_PPI0)
			printf("FIFO Underflow Error in PPI0 !!!\n\n");
		throughput_ppi0=(float)uiBytes*CCLK_SPEED/cycles_ppi0;
			printf("Throughput PPI0=%f MB/s\n\n", throughput_ppi0);
	#endif

	#ifdef ENABLE_MDMA1_DMA_800MBPS
		throughput_mdma1=(float)uiBytes*CCLK_SPEED/cycles_mdma1;
		printf("Throughput MDMA1=%f MB/s\n\n", throughput_mdma1);
	#endif

	#ifdef ENABLE_MDMA2_DMA_800MBPS
		throughput_mdma2=(float)uiBytes*CCLK_SPEED/cycles_mdma2;
		printf("Throughput MDMA2=%f MB/s\n\n", throughput_mdma2);
	#endif

	throughput_total = throughput_ppi0 +  throughput_mdma1 + throughput_mdma2;

	printf("Throughput total=%f MB/s\n\n", throughput_total);

	return 0;
}


void EPPI_init(unsigned int uiEppiID, unsigned int uiBytes, unsigned int EPPI_CTL_CFG, unsigned int EPPI_CTL2_CFG)
{
	//EPPI_Pin_Config(uiEppiID, 1, DATAWIDTH);									// Configure the pin multiplexing registers
	EPPI_Control_Config(uiEppiID, EPPI_CTL_CFG, EPPI_CTL2_CFG, 1);						// Configure EPPI control, control2 and clock divisor registers respectively.
	EPPI_Line_Config(uiEppiID, uiBytes, uiBytes, 0);								// Configure EPPI Line, Hcount and Hdelay registers respectively.
	EPPI_FSync_Config(uiEppiID, 3, uiBytes, 0 , 0 );								// Configure EPPI FS1-width, FS1-period, FS2-width and FS2-period registers respectively.
	EPPI_IMASK_Config(uiEppiID, 0);												// Configure EPPI IMASK register.

}



void EPPI0_Interrupt_Handler (uint32_t iid, void* handlerArg)
{
	clock_start_ppi0=clock_stop_ppi0;
	clock_stop_ppi0 = clock();
	cycles_ppi0_pre=cycles_ppi0;
	cycles_ppi0=clock_stop_ppi0-clock_start_ppi0;
	if(DMA_Count_PPI0>0)
	{
		cycles_ppi0=(DMA_Count_PPI0*cycles_ppi0_pre+cycles_ppi0)/(DMA_Count_PPI0+1);
	}

	*pREG_DMA14_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit
	DMA_Count_PPI0++;
	if(DMA_Count_PPI0==1)
		*pREG_DMA14_STAT |= ENUM_EPPI_STAT_YFIFOERR;			 //Clear the W1C interrupt status bit

}


void MDMA1_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma1=clock_stop_mdma1;
	clock_stop_mdma1 = clock();
	cycles_mdma1_pre=cycles_mdma1;
	cycles_mdma1=clock_stop_mdma1-clock_start_mdma1;
	if(DMA_Count_MDMA1>1)
	{
		cycles_mdma1=(DMA_Count_MDMA1*cycles_mdma1_pre+cycles_mdma1)/(DMA_Count_MDMA1+1);
	}

	*pREG_DMA19_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_MDMA1++;
}

void MDMA2_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma2=clock_stop_mdma2;
	clock_stop_mdma2= clock();
	cycles_mdma2_pre=cycles_mdma2;
	cycles_mdma2=clock_stop_mdma2-clock_start_mdma2;
	if(DMA_Count_MDMA2>1)
	{
		cycles_mdma2=(DMA_Count_MDMA2*cycles_mdma2_pre+cycles_mdma2)/(DMA_Count_MDMA2+1);
	}

	*pREG_DMA21_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_MDMA2++;
}

